@@ -236,7 +236,8 @@ nautilus_file_background_read_desktop_settings (char **color,
 		(theme_name, desktop_theme_source, &default_color, &default_image_uri, &default_placement);
 
         if (prefs->wallpaper_enabled) {
-                if (prefs->wallpaper_filename != NULL) {
+                if (prefs->wallpaper_filename != NULL &&
+                    prefs->wallpaper_filename [0] != '\0') {
                         *image = gnome_vfs_get_uri_from_local_path (prefs->wallpaper_filename);
                 } else {
                         *image = g_strdup (default_image_uri);
@@ -426,6 +427,7 @@ make_root_pixmap (gint width, gint height)
 	Display *display;
         char *display_name;
 	Pixmap result;
+	GdkPixmap *gdk_pixmap;
 
 	gdk_flush ();
 
@@ -447,7 +449,11 @@ make_root_pixmap (gint width, gint height)
 
 	XCloseDisplay (display);
 
-	return gdk_pixmap_foreign_new (result);
+	gdk_pixmap = gdk_pixmap_foreign_new (result);
+	gdk_drawable_set_colormap (GDK_DRAWABLE (gdk_pixmap),
+				   gdk_drawable_get_colormap (gdk_get_default_root_window ()));
+
+	return gdk_pixmap;
 }
 
 /* Set the root pixmap, and properties pointing to it. We
@@ -538,9 +544,9 @@ image_loading_done_callback (EelBackground *background, gboolean successful_load
 	set_root_pixmap (pixmap);
 
 	background_window = background_get_desktop_background_window (background);
-	if (background_window != NULL) {
+	if (background_window != NULL &&
+            gdk_drawable_get_depth (background_window) == gdk_drawable_get_depth (pixmap))
 		gdk_window_set_back_pixmap (background_window, pixmap, FALSE);
-	}
 
         g_object_unref (pixmap);
 }
