@@ -30,13 +30,16 @@
 #include "nautilus-directory.h"
 #include "nautilus-file-attributes.h"
 #include "nautilus-file-utilities.h"
+#include "nautilus-icon-factory.h"
 #include "nautilus-file.h"
 #include "nautilus-metadata.h"
+#include "nautilus-program-choosing.h"
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-gnome-extensions.h>
 #include <eel/eel-stock-dialogs.h>
 #include <eel/eel-string.h>
 #include <eel/eel-xml-extensions.h>
+#include <eel/eel-vfs-extensions.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-util.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
@@ -107,7 +110,6 @@ nautilus_link_desktop_file_local_create (const char        *directory_uri,
 
 	g_return_val_if_fail (directory_uri != NULL, FALSE);
 	g_return_val_if_fail (name != NULL, FALSE);
-	g_return_val_if_fail (image != NULL, FALSE);
 	g_return_val_if_fail (target_uri != NULL, FALSE);
 
 	uri = g_strdup_printf ("%s/%s", directory_uri, name);
@@ -116,22 +118,29 @@ nautilus_link_desktop_file_local_create (const char        *directory_uri,
 				    "Encoding=UTF-8\n"
 				    "Name=%s\n"
 				    "Type=%s\n"
-				    "X-Nautilus-Icon=%s\n"
 				    "URL=%s\n"
-				    "\n",
+				    "%s%s\n",
 				    name,
 				    get_tag (type),
-				    image,
-				    target_uri);
-
+				    target_uri,
+				    image != NULL ? "X-Nautilus-Icon=" : "",
+				    image != NULL ? image : "");
+	
 	desktop_item = gnome_desktop_item_new_from_string (uri,
 							   contents,
 							   strlen (contents),
 							   0,
 							   NULL);
-	if (!gnome_desktop_item_save (desktop_item, NULL, FALSE, NULL)) {
+	if (!desktop_item) {
+		g_free (contents);
+		g_free (uri);
+		return FALSE;
+	}
+
+	if (!gnome_desktop_item_save (desktop_item, uri, TRUE, NULL)) {
 		gnome_desktop_item_unref (desktop_item);
 		g_free (contents);
+		g_free (uri);
 		return FALSE;
 	}
 
@@ -257,12 +266,36 @@ nautilus_link_desktop_file_local_get_link_type (const char *path)
 	return retval;
 }
 
+gboolean
+nautilus_link_desktop_file_local_is_utf8 (const char       *uri)
+{
+	char *contents;
+	int file_size;
+	gboolean retval;
+
+	if (eel_read_entire_file (uri,
+				  &file_size,
+				  &contents) != GNOME_VFS_OK) {
+		return FALSE;
+	}
+	
+	if (g_strstr_len (contents, file_size, "Encoding=UTF-8\n") != NULL) {
+		retval = TRUE;
+	} else {
+		retval = FALSE;
+	}
+
+	g_free (contents);
+	
+	return retval;
+}
+
+
+
 static char *
 nautilus_link_desktop_file_get_link_uri_from_desktop (GnomeDesktopItem *desktop_file)
 {
-	char *terminal_command;
 	const char *launch_string;
-	gboolean need_term;
 	const char *type;
 	char *retval;
 
@@ -278,15 +311,9 @@ nautilus_link_desktop_file_get_link_uri_from_desktop (GnomeDesktopItem *desktop_
 		if (launch_string == NULL) {
 			return NULL;
 		}
-
-		need_term = gnome_desktop_item_get_boolean (desktop_file, "Terminal");
-		if (need_term) {
-			terminal_command = eel_gnome_make_terminal_command (launch_string);
-			retval = g_strconcat ("command:", terminal_command, NULL);
-			g_free (terminal_command);
-		} else {
-			retval = g_strconcat ("command:", launch_string, NULL);
-		}
+		
+		launch_string = gnome_desktop_item_get_location (desktop_file);
+		retval = g_strconcat (NAUTILUS_DESKTOP_COMMAND_SPECIFIER, launch_string, NULL);
 	} else if (strcmp (type, "URL") == 0) {
 		/* Some old broken desktop files use this nonstandard feature, we need handle it though */
 		retval = g_strdup (gnome_desktop_item_get_string (desktop_file, "Exec"));
@@ -309,9 +336,9 @@ nautilus_link_desktop_file_get_link_name_from_desktop (GnomeDesktopItem *desktop
 static char *
 nautilus_link_desktop_file_get_link_icon_from_desktop (GnomeDesktopItem *desktop_file)
 {
+	GnomeIconLoader *icon_loader;
 	char *icon_uri;
 	char *absolute;
-	char *icon_name;
 
 	icon_uri = g_strdup (gnome_desktop_item_get_string (desktop_file, "X-Nautilus-Icon"));
 	if (icon_uri != NULL) {
@@ -319,19 +346,11 @@ nautilus_link_desktop_file_get_link_icon_from_desktop (GnomeDesktopItem *desktop
 	}
 
 	/* Fall back to a standard icon. */
-	icon_name = g_strdup (gnome_desktop_item_get_string (desktop_file, "Icon"));
-	if (icon_name == NULL) {
-		return NULL;
-	}
+	icon_loader = nautilus_icon_factory_get_icon_loader ();
+	absolute = gnome_desktop_item_get_icon (desktop_file, icon_loader);
 	
-	if (icon_name[0] == '/') {
-		absolute = icon_name;
-	} else {
-		absolute = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, icon_name, TRUE, NULL);
-		g_free (icon_name);
-		if (absolute == NULL) {
-			return NULL;
-		}
+	if (absolute == NULL) {
+		return NULL;
 	}
 
 	icon_uri = gnome_vfs_get_uri_from_local_path (absolute);
